import 'dart:async';
import 'dart:math';

import 'package:minehunter/provider/data.dart';

class BoardAnimate {
  // Start game animation: blinking tiles
  static List<List<List<bool>>> createStartGameAnimationPatterns(Data myProvider) {
    final List<List<List<bool>>> patterns = [];

    const int patternsCount = 4;
    final int sizeHorizontal = myProvider.sizeHorizontal;
    final int sizeVertical = myProvider.sizeVertical;

    for (int patternIndex = 0; patternIndex < patternsCount; patternIndex++) {
      final List<List<bool>> pattern = [];
      for (int row = 0; row < sizeVertical; row++) {
        final List<bool> patternRow = [];
        for (int col = 0; col < sizeHorizontal; col++) {
          patternRow.add(((patternIndex + row + col) % 2 == 0));
        }
        pattern.add(patternRow);
      }
      patterns.add(pattern);
    }

    return patterns;
  }

  // Failed game animation: explosions blowing from exploded mines
  static List<List<List<bool>>> createExplosionAnimationPatterns(Data myProvider) {
    final List<List<List<bool>>> patterns = [];

    final int sizeHorizontal = myProvider.sizeHorizontal;
    final int sizeVertical = myProvider.sizeVertical;

    final List<List<int>> explodedMines = [];
    for (int row = 0; row < sizeVertical; row++) {
      for (int col = 0; col < sizeHorizontal; col++) {
        if (myProvider.cells[row][col].isExploded) {
          explodedMines.add([row, col]);
        }
      }
    }
    if (explodedMines.isEmpty) {
      explodedMines.add([(sizeVertical / 2).floor(), (sizeHorizontal / 2).floor()]);
    }

    final int patternsCount = max(sizeHorizontal, sizeVertical);

    for (int patternIndex = 0; patternIndex < patternsCount; patternIndex++) {
      final List<List<bool>> pattern = [];
      for (int row = 0; row < sizeVertical; row++) {
        final List<bool> patternRow = [];
        for (int col = 0; col < sizeHorizontal; col++) {
          bool isHilighted = false;
          for (int mineIndex = 0; mineIndex < explodedMines.length; mineIndex++) {
            double distance = sqrt(pow((explodedMines[mineIndex][0] - row), 2) +
                pow((explodedMines[mineIndex][1] - col), 2));
            isHilighted = isHilighted || ((patternIndex + distance) % 4 < 2);
          }
          patternRow.add(isHilighted);
        }
        pattern.add(patternRow);
      }
      patterns.add(pattern);
    }

    return patterns;
  }

  // Win game animation: rotating rays from center
  static List<List<List<bool>>> createWinGameAnimationPatterns(Data myProvider) {
    final List<List<List<bool>>> patterns = [];

    const int patternsCount = 20;
    final int sizeHorizontal = myProvider.sizeHorizontal;
    final int sizeVertical = myProvider.sizeVertical;

    for (int patternIndex = 0; patternIndex < patternsCount; patternIndex++) {
      List<List<bool>> pattern = [];
      for (int row = 0; row < sizeVertical; row++) {
        List<bool> patternRow = [];
        for (int col = 0; col < sizeHorizontal; col++) {
          double angle = 2 * atan2((sizeVertical / 2) - col, (sizeHorizontal / 2) - row);
          patternRow.add((angle + patternIndex / 3) % 2 < 1);
        }
        pattern.add(patternRow);
      }
      patterns.add(pattern);
    }

    return patterns;
  }

  // Default multi-purpose animation: sliding stripes, from top left to right bottom
  static List<List<List<bool>>> createDefaultAnimationPatterns(Data myProvider) {
    final List<List<List<bool>>> patterns = [];

    const int patternsCount = 16;
    final int sizeHorizontal = myProvider.sizeHorizontal;
    final int sizeVertical = myProvider.sizeVertical;

    for (int patternIndex = 0; patternIndex < patternsCount; patternIndex++) {
      final List<List<bool>> pattern = [];
      for (int row = 0; row < sizeVertical; row++) {
        final List<bool> patternRow = [];
        for (int col = 0; col < sizeHorizontal; col++) {
          patternRow.add(((patternIndex + row + col) % 4 == 0));
        }
        pattern.add(patternRow);
      }
      patterns.add(pattern);
    }

    return patterns;
  }

  static void startAnimation(Data myProvider, String animationType) {
    List<List<List<bool>>> patterns = [];

    switch (animationType) {
      case 'start':
        patterns = createStartGameAnimationPatterns(myProvider);
        break;
      case 'win':
        patterns = createWinGameAnimationPatterns(myProvider);
        break;
      case 'fail':
        patterns = createExplosionAnimationPatterns(myProvider);
        break;
      default:
        patterns = createDefaultAnimationPatterns(myProvider);
    }

    int patternIndex = patterns.length;

    myProvider.updateAnimationInProgress(true);

    const interval = Duration(milliseconds: 200);
    Timer.periodic(
      interval,
      (Timer timer) {
        if (patternIndex == 0) {
          timer.cancel();
          myProvider.resetAnimatedBackground();
          myProvider.updateAnimationInProgress(false);
        } else {
          patternIndex--;
          myProvider.setAnimatedBackground(patterns[patternIndex]);
        }
      },
    );
  }
}
